//=========================================================================
//Copyright LizardOfOz.
//This is still beta and things might and will change over time.
//
//Check the following links for updates:
//   https://tf2maps.net/members/lizardofoz.39426/#resources
//   https://twitter.com/LizardOfOz_
//   https://youtube.com/LizardOfOz
//
//Credits:
//  LizardOfOz - the original VSH Plugin. VSH Vscript recreation.
//  Maxxy - Saxton Hale's model imitating Jungle Inferno SFM; Custom animations and promotional material.
//  Velly - VFX, animations scripting, technical assistance
//  JPRAS - Mayann Hale body reference, model feedback.
//  MegapiemanPHD - Saxton Hale and Gray Mann voice acting.
//  Yakibomb - give_tf_weapon script bundle (used for Hale's first-person hands model).
//=========================================================================

isRoundSetup <- true
isRoundOver <- false;
starterClass <- {}
isFeedbackRound <- false;
roundStartTime <- Time();

function StartNewRound(void)
{
    if (isFeedbackRound || IsInWaitingForPlayers() || GetTotalPlayerCount() < 2)
        return;

    Convars.SetValue("mp_bonusroundtime", GetPersistentVar("mp_bonusroundtime"));

    local hale = GetHale();
    if (hale != null)
    {
        local wearable = null;
        while (wearable = Entities.FindByClassname(wearable, "tf_wearable*"))
            if (wearable.GetOwner() == hale)
                wearable.Kill();
    }

    ProgressHaleQueue();

    local forceRespawn = Entities.FindByClassname(null, "game_forcerespawn");
    EntFireByHandle(forceRespawn, "ForceRespawn", "", 0, null, null);

    foreach (player in GetAllValidMercs())
    {
        player.SwitchTeam(TF_TEAM.MERCS);
        player.ForceRegenerateAndRespawn();
        player.KeyValueFromString("targetname", "");
        TrySpawnAsMerc(player);
    }
    TrySpawnAsHale(GetHale());

    RunWithDelay("PlayRoundStartVO()", null, 2);
}
AddListener("round_start", StartNewRound, 0);

function PlayRoundStartVO()
{
    if (RandomInt(1, 10) <= 4)
        PlayAnnouncerVO("saxton_hale", "round_start_long");
    else
    {
        if (beerLines && RandomInt(1, 10) <= 4)
            PlayAnnouncerVO("saxton_hale", "round_start_beer")
        else
            PlayAnnouncerVO("saxton_hale", "round_start")

        local countdownDelay = setupLength - 8;
        RunWithDelay("PlayAnnouncerVO(`saxton_hale`, `count5`)", null, countdownDelay++);
        RunWithDelay("PlayAnnouncerVO(`saxton_hale`, `count4`)", null, countdownDelay++);
        RunWithDelay("PlayAnnouncerVO(`saxton_hale`, `count3`)", null, countdownDelay++);
        RunWithDelay("PlayAnnouncerVO(`saxton_hale`, `count2`)", null, countdownDelay++);
        RunWithDelay("PlayAnnouncerVO(`saxton_hale`, `count1`)", null, countdownDelay);
    }
}

function FinishRoundSetup()
{
    //Respawn all Mercs when Setup ends
    foreach (player in GetAllValidMercs())
    {
        starterClass[player] <- player.GetPlayerClass();
        if (IsMercDead(player))
            player.ForceRespawn();
    }
    TickHaleSetupStats(GetHale());
    isRoundSetup = false;

    local respawnRoom = null;
    while (respawnRoom = Entities.FindByClassname(respawnRoom, "func_respawnroom"))
        respawnRoom.Kill();

    local gamerules = Entities.FindByClassname(null, "tf_gamerules");
    SetPropInt(gamerules, "m_iRoundState", 7);
    SetPropInt(gamerules, "m_nHudType", 2);
}

function ProgressHaleQueue()
{
    local nextHale = GetPersistentVar("next_hale_player");
    SetPersistentVar("next_hale_player", null);
    if (IsValidPlayer(nextHale))
        SetHale(nextHale);
    else
        SetHale(RollNewHale(GetPersistentVar("last_hale_player")));
}

function RollNewHale(ignoredPlayer)
{
    local players = [];
    foreach (player in GetAllValidPlayers())
        if (player != ignoredPlayer)
            players.push(player);

    if (players.len() > 0)
        return players[RandomInt(0, players.len() - 1)];

    return Entities.FindByClassname(null, "player");
}

function HasRoundEnded(interval)
{
    if (IsInWaitingForPlayers() || GetTotalPlayerCount() < 2)
        return;
    if (GetAliveMercCount() <= 0)
        EndRound(TF_TEAM.HALE);
    local hale = GetHale();
    if (!IsRoundSetup() && (IsNotValidPlayer(hale) || IsPlayerDead(hale)))
        EndRound(TF_TEAM.MERCS);
}
AddListener("tick_always", HasRoundEnded, 1);

function EndRound(winner)
{
    if (isRoundOver)
        return;
    if (IsNotValidPlayer(GetHale()))
        winner = TF_TEAM.UNASSIGNED;

    local voiceLine;
    if (winner == TF_TEAM.HALE)
        voiceLine = "hale_win";
    else if (winner == TF_TEAM.MERCS)
        voiceLine = "mercs_win";
    else
        voiceLine = "bailout";
    RunWithDelay("PlayAnnouncerVO(`saxton_hale`, `"+voiceLine+"`)", null, 1);

    local roundWin = Entities.FindByClassname(null, "game_round_win");
    if (roundWin == null)
    {
        roundWin = SpawnEntityFromTable("game_round_win",
        {
            win_reason = "0",
            force_map_reset = "1", //not having
            TeamNum = "0",         //these 3 lines
            switch_teams = "0"     //causes the crash when trying to fire game_round_win
        });
    }
    EntFireByHandle(roundWin, "SetTeam", "" + winner, 0, null, null);
    EntFireByHandle(roundWin, "RoundWin", "", 0, null, null);
    printl(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>ass<<<<<<<<<<<")

    DoEntFire("vsh_round_end*", "Trigger", "", 0, null, null);
    if (winner == TF_TEAM.MERCS)
        DoEntFire("vsh_mercs_win*", "Trigger", "", 0, null, null);
    else
        DoEntFire("vsh_hale_win*", "Trigger", "", 0, null, null);
    isRoundOver = true;
}