//=========================================================================
//Copyright LizardOfOz.
//This is still beta and things might and will change over time.
//
//Check the following links for updates:
//   https://tf2maps.net/members/lizardofoz.39426/#resources
//   https://twitter.com/LizardOfOz_
//   https://youtube.com/LizardOfOz
//
//Credits:
//  LizardOfOz - the original VSH Plugin. VSH Vscript recreation.
//  Maxxy - Saxton Hale's model imitating Jungle Inferno SFM; Custom animations and promotional material.
//  Velly - VFX, animations scripting, technical assistance
//  JPRAS - Mayann Hale body reference, model feedback.
//  MegapiemanPHD - Saxton Hale and Gray Mann voice acting.
//  Yakibomb - give_tf_weapon script bundle (used for Hale's first-person hands model).
//=========================================================================

::haleChargeAbilityInstance <- null;

class ChargeHaleAbility extends HaleChange
{
    meter = 0;
    isCurrentlyDashing = false;
    haleLastFrameDownVelocity = 0;
    voiceTime = 0;
    voiceRNG = "a";
    bashedByHale = [];
    midAirWindUpOverload = 0;

    function OnApply()
    {
        haleChargeAbilityInstance = this;
    }

    function MeterValueAsString()
    {
        if (meter == 0)
            return "Ready";
        else if (meter < 0)
            return format("%.1fs", -meter);
        else
            return format("%d%%", meter * 100);
    }

    function OnTick(interval)
    {
        if (isCurrentlyDashing)
            return;

        if (meter < 0)
        {
            meter += interval;
            if (meter > 0)
            {
                EmitSoundOnClient("TFPlayer.ReCharged", hale);
                meter = 0;
            }
            else
                return;
        }
    }

    function OnFrameTick()
    {
        if (isCurrentlyDashing)
        {
            TickDash();
            return;
        }
        if (meter < 0)
            return;

        local buttons = GetPropInt(hale, "m_nButtons");
        if (hale.IsUsingActionSlot()
            || (buttons & Constants.FButtons.IN_ATTACK3)
            || (buttons & Constants.FButtons.IN_RELOAD))
        {
            WindUp();
        }
        else if (meter >= 0.01)
            Perform();
    }

    function WindUp()
    {
        meter += 0.01;
        if (meter >= 1.0)
        {
            meter = 1.0;
            if (!(hale.GetFlags() & Constants.FPlayer.FL_ONGROUND))
            {
                midAirWindUpOverload += 0.01;
                if (midAirWindUpOverload >= 1)
                {
                    Perform();
                    return;
                }
            }
            else
                midAirWindUpOverload = 0;
        }
        if (!(hale.GetFlags() & Constants.FPlayer.FL_ONGROUND))
            SetPropInt(hale.GetActiveWeapon(), "m_AttributeManager.m_Item.m_iItemDefinitionIndex", 42);
        HalePlayViewModelAnim("vsh_dash_windup");
        hale.AddCustomAttribute("no_attack", 0.25, 1);
        if (Time() > voiceTime)
        {
            voiceRNG = ["a","b","c"][RandomInt(0,2)];
            voiceTime = Time() + 999;
            EmitPlayerVO(hale, "charge_"+voiceRNG);
            EmitSoundOn("VSH.ChargeSFX", hale);
            local windUpVFX = Entities.FindByName(null, "windup_vfx");
            if (windUpVFX == null)
            {
                windUpVFX = hale.CreateCustomWearable(null, SAXTON_HALE_WINDUP_MODEL_PATH);
                windUpVFX.KeyValueFromString("targetname", "windup_vfx");
            }
        }
        hale.SetAbsVelocity(Vector(0,0,0));
        hale.AddCustomAttribute("no_attack", 1, 0.5);
        hale.AddCustomAttribute("move speed penalty", 0.25, 0.5);
        if (!hale.InCond(Constants.ETFCond.TF_COND_AIMING))
            hale.AddCond(Constants.ETFCond.TF_COND_AIMING);
    }

    function Perform()
    {
        DoEntFire("windup_vfx", "Kill", "", 0, null, null);
        SetPropInt(hale.GetActiveWeapon(), "m_AttributeManager.m_Item.m_iItemDefinitionIndex", 5); //todo future
        isCurrentlyDashing = true;
        bashedByHale = [];
        hale.RemoveCond(Constants.ETFCond.TF_COND_AIMING);
        hale.AddCondEx(Constants.ETFCond.TF_COND_HALLOWEEN_KART_DASH, 0.1, hale);

        local haleForwardDirection = hale.EyeAngles().Forward();
        local forwardOffset = haleForwardDirection * 60;
        local chargeZone = Entities.FindByName(null, "hale_charge_zone");
        chargeZone.SetAbsOrigin(hale.GetCenter() + forwardOffset);

        DoEntFire("hale_charge_zone", "Enable", "", 0, hale, hale);
        DoEntFire("hale_charge_catapult", "Enable", "", 0, hale, hale);


        HalePlayViewModelAnim("vsh_dash_loop");

        local chargeDuration = meter / 1.66 + 0.4;

        local forward = hale.EyeAngles();
        local pitch = hale.GetFlags() & Constants.FPlayer.FL_ONGROUND ? forward.Pitch() - 10 : forward.Pitch();
        forward = QAngle(pitch, forward.Yaw(), 0);
        hale.Yeet(forward.Forward() * 800 * chargeDuration)
        hale.SetGravity(0.3);
        voiceTime = 0;
        midAirWindUpOverload = 0;

        hale.AddCondEx(Constants.ETFCond.TF_COND_SHIELD_CHARGE, chargeDuration, null);
        hale.AddCondEx(Constants.ETFCond.TF_COND_KNOCKED_INTO_AIR, chargeDuration, null);
        hale.AddCustomAttribute("no_attack", 1, chargeDuration);
        EmitPlayerVO(hale, "dash_"+voiceRNG);
        EmitSoundOn("VSH.DashSFX", hale);
        RunWithDelay("haleChargeAbilityInstance.Finish()", null, chargeDuration);

        local dashDome = hale.CreateCustomWearable(null, SAXTON_HALE_DASH_MODEL_PATH);
        EntFireByHandle(dashDome, "Kill", "", chargeDuration, null, null);
    }

    function Finish()
    {
        local hale = GetHale();
        HalePlayViewModelAnim("vsh_dash_end");
        hale.AddCondEx(Constants.ETFCond.TF_COND_GRAPPLINGHOOK_LATCHED, 0.1, hale);
        meter = -10;
        isCurrentlyDashing = false;
        hale.SetGravity(1);
        DoEntFire("hale_charge_zone", "Disable", "", 0, hale, hale)
        DoEntFire("hale_charge_catapult", "Disable", "", 0, hale, hale)
    }

    function TickDash()
    {
        local haleForwardDirection = hale.EyeAngles().Forward();
        SetPropInt(hale, "m_Shared.m_bJumping", 1);
        SetPropFloat(hale, "m_Shared.m_flJumpTime", 1);

        SetPropEntity(hale, "m_hGroundEntity", null);
        local force = 1400;
        hale.SetAbsVelocity(haleForwardDirection*force)

        local catapult = Entities.FindByName(null, "hale_charge_catapult");
        catapult.SetAbsOrigin(hale.GetCenter());

        local forwardOffset = haleForwardDirection * 60;
        local chargeZone = Entities.FindByName(null, "hale_charge_zone");
        chargeZone.SetAbsOrigin(hale.GetCenter() + forwardOffset);

        local building = null;
        while (building = Entities.FindByClassname(building, "obj_*"))
        {
            if ((hale.GetCenter() - building.GetOrigin()).LengthSqr() < 10000)
            building.TakeDamageEx(hale, hale, hale.GetActiveWeapon(), Vector(0,0,0), hale.GetOrigin(), 100, 1);
        }
    }

    function BashTarget(victim)
    {
        local hale = GetHale();
        if (IsNotHale(hale) || IsHale(victim) || bashedByHale.find(victim) != null)
            return;

        bashedByHale.push(victim);
        EmitSoundOn("DemoCharge.HitFlesh", victim);

        local forward = victim.GetCenter() - hale.GetCenter();
        forward.Norm();
        forward.x = forward.x * 1250
        forward.y = forward.y * 1250
        forward.z = 450;
        victim.Yeet(forward);

        victim.TakeDamageEx(hale, hale, hale.GetActiveWeapon(), forward, hale.GetOrigin(), victim.GetMaxHealth() * 0.55, 1);
    }
}
characterChangesClasses.push(ChargeHaleAbility);